# R functions related to the rescaled cofitness analysis.

# od, media, and readingMethod are vectors
# only defined media (with _noCarbon or _noNitrogen), LB, marine_broth_2216, R2A, or PYE are recognized
# readingMethod may be "standard_cuvette" (which need not be calibrated) and may have a _Transparent suffix, i.e.
#	24well_Transparent
# odcalib must contain Media, GrowthFormat (i.e. 96well), MicroplateOD, and StandardOD
CalibrateOD = function(od, media, readingMethod, odcalib, pastmax=TRUE) {
	stopifnot(all(media %in% odcalib$Media));
        readingMethod = sub("standard_cuvette", "", readingMethod);
        readingMethod = sub("_Transparent", "", readingMethod);
	stopifnot(all(readingMethod %in% odcalib$GrowthFormat | readingMethod == ""));

	by = paste(media, "_", readingMethod);
        d = split(data.frame(od=od, media=media, readingMethod=readingMethod), by);
	calibavg = aggregate(odcalib[,"MicroplateOD",drop=F], odcalib[,c("Media","GrowthFormat","StandardOD")], mean);
        calibsplit = split(calibavg, paste(calibavg$Media, calibavg$GrowthFormat));
        withnorm = lapply(d, function(data) {
          data$normod = data$od;
	  if (data$readingMethod[1] == "")  return(data);
          index = paste(data$media[1], data$readingMethod[1]);
          stopifnot(index %in% names(calibsplit));
	  calib = calibsplit[[ index ]];
          stopifnot(nrow(calib) > 1);
          stopifnot(all(!is.na(calib$MicroplateOD)));
          stopifnot(all(!is.na(calib$StandardOD)));
	  data$normod = approx(calib$MicroplateOD, calib$StandardOD, xout=data$od, rule=2)$y;
	  u = data$od > max(calib$MicroplateOD);
	  if(pastmax && any(u)) {
	    top = tail(calib[order(calib$MicroplateOD),], n=3);
            model = lm(StandardOD ~ MicroplateOD, top);
	    data$normod[u] = predict(model, data.frame(MicroplateOD = data$od[u]));
	  }
          return(data);
	});
 	return(unsplit(withnorm, by)$normod);
}

TopCofit = function(g, r,
		debug = FALSE,
		fraction=0.02,
		n = pmin(pmax(1,round(length(g)*fraction)),length(g)-1)) {
	if (length(g) != nrow(r)) stop("rows and number of genes does not match");
	cofits = cor(t(r)); # correlation between rows; note does not check for NA (is much faster that way)
	if(debug) cat("nTop",n,"\n");
	nOut = length(g) * n;
	if(debug) cat("Making output with",nOut,"rows\n");
        out_hitId = rep("", nOut);
        out_cofit = rep(NA, nOut);
	for (i in 1:length(g)) {
		values = cofits[i,];
		j = order(-values)[2:(n+1)]; # assume self is in position 1
		outi = (i-1)*n + (1:n); # where to put inside out
		out_hitId[outi] = g[j];
		out_cofit[outi] = values[j];
	}
	out = data.frame(locusId=rep(g,each=n), hitId=out_hitId, cofit=out_cofit, rank=rep(1:n,length(g)));
	return(out);
}
